
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




require 'upload_module'
require 'partner_search_box'

# 請求書
class InvoicesController < ApplicationController
  include UploadModule
  before_filter :login_required
  before_filter :role_keiri_required, :only => [:sign_off_list, :sign_off]
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update, :print_out,
                                      :change_partner_do, :cancel_invoice ],
         :redirect_to => { :action => :list }
  
  # （自部門の）請求書の一覧
  def list
    div = current_user.default_division
    raise "current_user has not any division." if !div
    @cdiv = div.default_controller
    raise if !@cdiv
    @invoices = Invoice.paginate :page => params[:page], :per_page => 25,
               :conditions => ["division_id IN (SELECT division_id
                                FROM controllers_divisions
                                WHERE control_division_id = ?)", @cdiv.id],
               :order => "buy_day DESC"
    
    # TODO: こっちもpaginate
    @borne_invoices = Invoice.find_by_sql [<<EOF, @cdiv.id]
SELECT invoices.* FROM invoice_details
            JOIN invoices ON invoice_details.invoice_id = invoices.id
    WHERE invoice_details.division_id
               IN (SELECT division_id FROM controllers_divisions
                                      WHERE control_division_id = ?)
EOF
  end

  # 承認を要するリスト
  def sign_off_list
    @invoices = Invoice.paginate :page => params[:page], :per_page => 40,
             :conditions => ['state < ?', Invoice::SIGNED_OFF],
             :order => 'due_day'
  end
  
  # 承認
  def sign_off
    @invoice = Invoice.find(params[:id])
    Invoice.transaction {
      do_sign_off
    }
    flash[:notice] = '請求書を承認しました。'
    if params[:prev] &&  params[:prev] == 'signoff'
      redirect_to :action => 'sign_off_list'
    else
      redirect_to :action => 'show', :id => @invoice
    end
  end
  
  def do_sign_off
    if @invoice.state < Invoice::SIGNED_OFF
      il = InvoiceLog.new(:invoice => @invoice,
                          :kind => InvoiceLog::SIGN_OFF,
                          :remarks => "",
                          :done_by => current_user,
                          :done_at => Time.now)
      il.save!
      @invoice.state = Invoice::SIGNED_OFF
      @invoice.save!
    end
  end
  private :do_sign_off
  
  # 詳細画面
  def show
    @invoice = Invoice.find(params[:id])
    @prev = params[:prev]
  end
  
  # 新しい請求書の画面
  def new
    @invoice = Invoice.new
    @partner = Partner.find(params[:partner])
  end
  
  # 作成
  def create
    raise "current_user has not any division." if !current_user.default_division

    @partner = Partner.find(params[:id])
    @invoice = Invoice.new params[:invoice]
    @invoice.attributes = {
        :partner => @partner,
        :division => current_user.default_division,
        :created_by => current_user,
        :state => 0,
        :payment_state => 0
    }
    @invoice.bank_account_id = nil if params[:no_bank]
    @invoice.invoice_no = nil if @invoice.invoice_no.to_s.strip == ''

    Invoice.transaction {
      details = create_details(@invoice.currency.exponent)
      details.each {|r| @invoice.invoice_details << r}
      if @invoice.save
        details.each {|r| r.save! }

        flash[:notice] = '請求書は正常に登録されました。支払依頼を出力してください。'
        if params[:prev] && params[:prev] == 'list'
          redirect_to :action => 'list'
        else
          redirect_to :action => 'show', :id => @invoice
        end
      else
        render :action => 'new', :partner => @partner
      end
    } # of transaction
  end

  # 編集画面
  def edit
    @invoice = Invoice.find(params[:id])
    @partner = @invoice.partner
    if !@invoice.bank_account ||
       !@partner.bank_accounts.collect {|r|
          r.id}.include?(@invoice.bank_account.id) # 銀行口座の紐付けを変更してる
      @no_bank = true
    end
  end

  # 既存のデータの更新
  def update
    # まず、フォームデータが妥当か確認する
    orig_invoice = Invoice.find params[:id]
    @partner = orig_invoice.partner
    @invoice = Invoice.new params[:invoice]   # 検証用のダミーレコード
    details = create_details(@invoice.currency.exponent)
    details.each {|r| @invoice.invoice_details << r}

    if !@invoice.valid?
      @invoice.id = orig_invoice.id
      render :action => 'edit', :id => @invoice
      return
    end

    # 更新する
    begin
      Invoice.transaction {
        old_details = orig_invoice.invoice_details.collect {|r| r}
        @invoice = orig_invoice
        @invoice.attributes = params[:invoice]
        @invoice.bank_account_id = nil if params[:no_bank]
        @invoice.invoice_no = nil if @invoice.invoice_no.to_s.strip == ''

        @invoice.save!
        old_details.each {|r| r.destroy } # 詳細行は作り直す
        details.each {|r| r.invoice = @invoice; r.save! }

        il = InvoiceLog.new(:invoice => @invoice,
                          :kind => InvoiceLog::MODIFY,
                          :remarks => params[:message] || "",
                          :done_by => current_user,
                          :done_at => Time.now)
        il.save!

        # 同時に承認
        if !params[:update_approval].blank?
          do_sign_off
        end
      }
    rescue ActiveRecord::StaleObjectError
      @invoice = Invoice.find params[:id]
      @invoice.errors.add_to_base 'ほかで同時に更新されました。'
      render :action => 'edit', :id => @invoice
      return 
    end

    flash[:notice] = '請求書は正常に更新されました。'
    if params[:prev] && params[:prev] == 'signoff'
      redirect_to :action => 'sign_off_list'
    else
      redirect_to :action => 'show', :id => @invoice
    end
  end

  # 明細行のレコードを作る
  def create_details(new_exp)
    result = []
    last = nil
    detail_lines = params[:lines].to_i
    raise if detail_lines == 0
    (0..(detail_lines - 1)).each {|i|      
      if (pa = params["detail#{i}"])[:amount].to_f != 0.0
        ac = pa[:account_title_id].to_i != 0 ? AccountTitle.find(pa[:account_title_id]) : last.account_title
        detail = InvoiceDetail.new(
          :invoice => @invoice,
          :account_title => ac,
          :division => Division.find(pa[:division_id]),
          :remarks => pa[:remarks],
          :vat_code => ac.get_real_vat_code(pa[:vat_code])
        )
        detail.amount = (pa[:amount].to_f * (10 ** new_exp) + 0.5).floor
        last = detail
        result << detail
      end
    }
    return result
  end
  private :create_details

  # 削除
  def destroy
    inv = Invoice.find params[:id]
    raise if inv.state >= 20

    partner = inv.partner
    Invoice.transaction {
      InvoiceLog.delete_all ['invoice_id = ?', inv.id]
      InvoiceDetail.delete_all ['invoice_id = ?', inv.id]
      inv.destroy
    }
    flash[:notice] = '請求書を削除しました。'
    if params[:prev] && params[:prev] == 'list'
      redirect_to :action => 'list'
    elsif params[:prev] && params[:prev] == 'signoff'
      redirect_to :action => 'sign_off_list'
    else
      redirect_to :controller => 'partners', :action => 'show', :id => partner
    end
  end
  
  # キャンセル. 会計システム連動済みのときはこっち
  def cancel_invoice
    inv = Invoice.find params[:id]
    Invoice.transaction {
      inv.payment_state = 20
      inv.save!
      il = InvoiceLog.new(:invoice => inv,
                          :kind => InvoiceLog::CANCEL,
                          :remarks => params[:message] || "",
                          :done_by => current_user,
                          :done_at => Time.now)
      il.save!
    }
    flash[:notice] = '請求書をキャンセルしました。'
    redirect_to :controller => 'partners', :action => 'show', :id => inv.partner
  end

  # コピーして新しい請求書を起票
  def copy_new
    @invoice = Invoice.find(params[:id])
    @invoice.buy_day = ((@invoice.buy_day + 1) >> 1) - 1
    @invoice.due_day = ((@invoice.due_day + 1) >> 1) - 1
    @partner = @invoice.partner
    @no_bank = !@invoice.bank_account
    render :action => 'new'
  end

  # 支払依頼を印刷するリスト
  def payment_list
    # 自分が起票したもののみ
    @invoices = Invoice.find :all,
                    :conditions => ['created_by = ? AND payment_state = 0',
                                    current_user.id]
  end
  
  # プレビュー画面の表示 & 確定
  def print_out
    @invoices = Invoice.find params[:pp], :order => 'partner_id, due_day'
    render :action => 'print_out'
    Invoice.transaction {
      @invoices.each {|inv|
        inv.payment_state = 10  # 印刷済み
        inv.save!
      }
    }
  end
  
  # 支払先を変更する画面
  def change_partner
    @invoice = Invoice.find params[:id]
  end
  
  # 支払先を変更する2
  def change_partner2
    @invoice = Invoice.find params[:id]

    @search = PartnerSearchBox.new(params[:search])
    @partners = @search.find_by_text
  end
  
  def change_partner_do
    @invoice = Invoice.find params[:id]
    @partner = Partner.find params[:partner]
    Invoice.transaction {
      @invoice.partner = @partner
      @invoice.bank_account = nil
      il = InvoiceLog.new(:invoice => @invoice,
                          :kind => InvoiceLog::MODIFY,
                          :remarks => '取引先を変更。',
                          :done_by => current_user,
                          :done_at => Time.now)
      il.save
      if @invoice.save
        flash[:notice] = '請求書は正常に更新されました。'
        redirect_to :action => 'show', :id => @invoice
      else
        raise
      end
    }
  end
  
  # ログのremarksを更新する
  def up_remarks
    il = InvoiceLog.find params[:id]
    @invoice = il.invoice
    il.remarks = params[:last_log]
    if il.save
      flash[:notice] = 'remarksを更新しました。'
      redirect_to :action => 'show', :id => @invoice
    else
      raise
    end
  end
  
  # 関連する輸入報告
  def related_import
    @invoice = Invoice.find params[:id]
  end
  
  def related_import2
    @invoice = Invoice.find params[:id]
    text = params[:search][:text]
    raise if text.to_s.strip == '' # TODO: fixme
    text = text.gsub(/[ \t]/, '')
    @declarations = ImportDeclaration.find :all,
          :conditions => ['declaration_number = ? OR bl_number = ?', text, text]
  end
  
  # 実行
  def related_import3
    @invoice = Invoice.find params[:id]
    @decl = ImportDeclaration.find params[:decl]
    ii = ImportsInvoice.new :invoice => @invoice,
                            :import_declaration => @decl,
                            :created_by => current_user
    if ii.save
      flash[:notice] = '輸入報告を紐付けました。'
      redirect_to :action => 'show', :id => @invoice
    else
      raise
    end
  end
end
