
require 'digest/sha1'

# ユーザ (社員)
class User < ActiveRecord::Base
  has_many :users_divisions
  has_many :divisions, :through => :users_divisions
  
  # Virtual attribute for the unencrypted password
  attr_accessor :password
  attr_accessor :password_confirmation
  before_save :encrypt_password

  validates_presence_of :worker_code
  validates_uniqueness_of :worker_code, :message => "が重複しています。"
  validates_presence_of :name
  validates_presence_of :kana_name
  
  before_save :normalize_string
  
  def normalize_string
    self.name = normalize_kc(name)
    self.kana_name = normalize_kc(kana_name)
    
    self.login = nil if login.to_s.strip == ""
    self.email = nil if email.to_s.strip == ""
  end

  # @override
  def validate
    errors.add "worker_code", "は0以外でなければなりません。" if worker_code == 0
    if !login.blank?
      errors.add 'email', 'がありません。' if !email || email.strip == ''
      if !password.blank?
        errors.add 'password', 'が短すぎます。' if password.length < 3
        errors.add 'password_confirmation', 'が一致していません。' if password != password_confirmation
      end
    end
  end
  
  # @override
  def validate_on_create
    if !login.blank?
      errors.add 'login', 'が重複しています。' if User.find_by_login(login)
      errors.add 'email', 'が重複しています。' if User.find_by_email(email)
      errors.add 'password', 'がありません。' if password.blank?
    end
  end

  # @override
  def validate_on_update
    if !login.blank?
      if User.find(:first, :conditions => ['login = ? AND id <> ?', login, id])
        errors.add 'login', 'が重複しています。'
      end
      if User.find(:first, :conditions => ['email = ? AND id <> ?', email, id])
        errors.add 'email', 'が重複しています。' 
      end
    end
  end
  
  def roles
    Role.find :all,
        :select => 'roles.*',
        :joins => 'JOIN divisions ON roles.id = divisions.role_id JOIN users_divisions ON divisions.id = users_divisions.division_id',
        :conditions => ['users_divisions.user_id = ?', id] 
  end

  def has_role?(name)
    raise if name != :keiri # 今のところ経理だけ。
                            # TODO: 経理承認と全社閲覧を分割すること

    roles.each {|role|
      return true if role.keiri != 0
    }
    return false
  end

  # 主な所属部門
  def default_division
    ud = UsersDivision.find(:first,
                :conditions => ['user_id = ? AND default_pair <> 0', id])
    ud ? ud.division : nil
  end
  

  ###########################################################################
  # from acts_as_authenticated

  # Authenticates a user by their login name and unencrypted password.
  # Returns the user or nil.
  def self.authenticate(login, password)
    u = find :first, :conditions => ['login = ? AND active <> 0', login] # need to get the salt
    u && u.authenticated?(password) ? u : nil
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  protected
  # before filter 
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{login}--") if new_record?
    self.crypted_password = encrypt(password)
  end

  def password_required?
    crypted_password.blank? || !password.blank?
  end
  
  # from acts_as_authenticated
  ###########################################################################
end
